function sys = sdpvar(varargin)
%SDPVAR Create symbolic decision variable
%       
%   You can create a sdpvar variable by: 
%     X = SDPVAR(n)               Symmetric nxn matrix
%     X = SDPVAR(n,n)             Symmetric nxn matrix
%     X = SDPVAR(n,m)             Full nxm matrix
%
%   Definition of multiple scalars can be simplified 
%     SDPVAR x y z w
%
%   The parametrizations supported are
%     X = SDPVAR(n,n,'symmetric') Symmetric nxn matrix 
%     X = SDPVAR(n,n,'full')      Full nxn matrix
%     X = SDPVAR(n,n,'toeplitz')  Symmetric Toeplitz
%     X = SDPVAR(n,n,'hankel')    Symmetric Hankel
%     X = SDPVAR(n,n,'skew')      Skew-symmetric       
%
%   The letters 'sy','f','ha', 't' and 'sk' are searched for in the third argument   
%   hence sdpvar(n,n,'toeplitz') gives the same result as sdpvar(n,n,'t')
%
%   Only square Toeplitz and Hankel matries are supported
%
%   A scalar is defined as a 1x1 matrix
%
%   In addition to the matrix type, a fourth argument 
%   can be used to obtain a complex matrix. All the 
%   matrix types above apply to a complex matrix, and
%   in addition a Hermitian type is added
%
%     X = SDPVAR(n,n,'hermitian','complex')            
%
%   The other types are obtained as above
%     X = SDPVAR(n,n,'symmetric','complex') Complex symmetric nxn matrix
%     X = SDPVAR(n,n,'full','complex')      Complex full nxn matrix
%     ... and the same for Toeplitz, Hankel and skew-symmetric
%
%   See also SEE, @SDPVAR/SET, INTVAR, BINVAR, methods('sdpvar')

% Author Johan Lfberg
% $Id: sdpvar.m,v 1.10 2005/01/05 10:05:03 johanl Exp $

persistent varCount;
persistent optSolution;

% First time called, reset #free variables and solution
if isempty(varCount)
    varCount=0;
    optSolution.info = 'Initialized by YALMIP';
    optSolution.variables = [];
    optSolution.optvar  =[];
end

if nargin==0
    %help sdpvar
    return
end

% Check for commmand OR sdpvar x y z syntax
if ischar(varargin{1})
    switch varargin{1}   
        case 'clear'
            varCount=0;  
            optSolution.info = 'Initialized by YALMIP';
            optSolution.variables = [];
            optSolution.optvar  =[];
             return
        case 'nvars'
            sys = varCount;
             return
        case 'setAllSolution'
            optSolution.optvar = varargin{2}.optvar;
            optSolution.variables = varargin{2}.variables;
             return
        case 'setSolution'
            if isempty(optSolution.variables)
                optSolution = varargin{2};
            else
                % Just save some stuff first
                newSolution = varargin{2};
                oldSolution = optSolution;
                optSolution = varargin{2};
                keep_these = find(~ismember(oldSolution.variables,newSolution.variables));
                optSolution.optvar    = [oldSolution.optvar(keep_these);newSolution.optvar(:)];
                optSolution.variables = [oldSolution.variables(keep_these);newSolution.variables(:)]; 	    
            end
             return
        case 'getSolution'
            sys = optSolution;
             return
        case 'setnvars' % Hard-core, extremely private
            varCount = varargin{2};
             return
        otherwise 
            n = length(varargin);
            for k = 1:n
                if ~isvarname(varargin{k})
                    error('symbolic:sym:errmsg1','Not a valid variable name.')
                end
            end
            for k = 1:n
                varname = varargin{k};                
                assignin('caller',varname,sdpvar(1,1));
            end
            return
    end
end

% Supported matrix types
% - symm
% - full
% - skew
% - hank
% - toep
switch nargin
    case 1 %Bug in MATLAB 5.3!! sdpvar called from horzcat!!!!????
        if isempty(varargin{1})
            sys = varargin{1};
            return
        end
        n = varargin{1};
        m = varargin{1};
        if n*m==0
            sys = zeros(n,m);
            return
        end
        if (n==m)
            matrix_type = 'symm';
            nvar = n*(n+1)/2;
        else
            matrix_type = 'full';
            nvar = n*m;
        end 
    case 2
        n = varargin{1};
        m = varargin{2};
        if n*m==0
            sys = zeros(n,m);
            return
        end
        if (n==m)
            matrix_type = 'symm';
            nvar = n*(n+1)/2;
        else
            matrix_type = 'full';
            nvar = n*m;
        end
    case {3,4}
        n = varargin{1};
        m = varargin{2};
        if n*m==0
            sys = zeros(n,m);
            return
        end
        
        % Check for complex or real
        if (nargin == 4)
            if ~ischar(varargin{4})
                help sdpvar
                error('Fourth argument should be ''complex'' or ''real''')
            end            
            index_cmrl = strmatch(varargin{4},{'real','complex'});            
            if isempty(index_cmrl)
                error('Fourth argument should be ''complex'' or ''real''. See help above')
            end	
        else
            if ~ischar(varargin{3})
                help sdpvar
                error('Third argument should be ''symmetric'', ''full'', ''hermitian'',...See help above')
            end
            index_cmrl = 1;
        end;
        
        if isempty(varargin{3})
            index_type = 7; %Default symmetric
        else
            if ~isempty(strmatch(varargin{3},{'complex','real'}))
                % User had third argument as complex or real
                error(['Third argument should be ''symmetric'', ''full'', ''toeplitz''... Maybe you meant sdpvar(n,n,''full'',''' varargin{3} ''')'])
            end
            index_type = strmatch(varargin{3},{'toeplitz','hankel','symmetric','full','rhankel','skew','hermitian'});
        end
        
        if isempty(index_type)
            error(['Matrix type "' varargin{3} '" not supported'])
        else
            switch index_type+100*(index_cmrl-1)
                case 1
                    if n~=m
                        error('Toeplitz matrix must be square')
                    else
                        matrix_type = 'toep';
                        nvar = n;
                    end
                    
                case 2
                    if n~=m
                        error('Hankel matrix must be square')
                    else
                        matrix_type = 'hank';
                        nvar = n;
                    end
                    
                case 3
                    if n~=m
                        error('Symmetric matrix must be square')
                    else
                        matrix_type = 'symm';
                        nvar = n*(n+1)/2;
                    end
                    
                case 4
                    matrix_type = 'full';
                    nvar = n*m;
                    if nvar==1
                        matrix_type = 'symm';
                    end
                    
                case 5
                    if n~=m
                        error('Hankel matrix must be square')
                    else
                        matrix_type = 'rhankel';
                        nvar = 2*n-1;
                    end
                    
                case 6
                    if n~=m
                        error('Skew symmetric matrix must be square')
                    else
                        matrix_type = 'skew';
                        nvar = (n*(n+1)/2)-n;
                    end
                    
                case 7
                    if n~=m
                        error('Symmetric matrix must be square')
                    else
                        matrix_type = 'symm';
                        nvar = n*(n+1)/2;
                    end
                    
                case 101
                    if n~=m
                        error('Toeplitz matrix must be square')
                    else
                        matrix_type = 'toep complex';
                        nvar = 2*n;
                    end
                    
                case 102
                    if n~=m
                        error('Hankel matrix must be square')
                    else
                        matrix_type = 'hank complex';
                        nvar = (2*n);
                    end
                    
                case 103
                    if n~=m
                        error('Symmetric matrix must be square')
                    else
                        matrix_type = 'symm complex';
                        nvar = 2*n*(n+1)/2;
                    end
                    
                case 104
                    matrix_type = 'full complex';
                    nvar = 2*n*m;
                    if nvar==1
                        matrix_type = 'symm complex';
                    end
                    
                case 105
                    if n~=m
                        error('Hankel matrix must be square')
                    else
                        matrix_type = 'rhankel complex';
                        nvar = 2*(2*n-1);
                    end
                    
                case 106
                    if n~=m
                        error('Skew symmetric matrix must be square')
                    else
                        matrix_type = 'skew complex';
                        nvar = 2*((n*(n+1)/2)-n);
                    end
                    
                case 107
                    if n~=m
                        error('Hermitian matrix must be square')
                    else
                        matrix_type = 'herm complex';
                        nvar = n*(n+1)/2+(n*(n+1)/2-n);
                    end
                    
                    
                otherwise
                    error('Bug! Report!');
            end
            
        end
        
    case 5 % Fast version for internal use
        sys.basis = varargin{5};
        sys.lmi_variables=varargin{4};
        sys.n = varargin{1};
        sys.m = varargin{2};
        sys.typeflag = 0;
        sys.savedata = [];
        sys.extra = [];
        
        % Find zero-variables
        constants = find(sys.lmi_variables==0);
        if ~isempty(constants);
            sys.lmi_variables(constants)=[];
            sys.basis(:,1) = sys.basis(:,1) + sum(sys.basis(:,1+constants),2);
            sys.basis(:,1+constants)=[];
        end
        if isempty(sys.lmi_variables)
            sys = full(reshape(sys.basis(:,1),sys.n,sys.m));        
        else
            sys = class(sys,'sdpvar');
        end
        return
    case 6 % Fast version for internal use
        sys.basis = varargin{5};
        sys.lmi_variables=varargin{4};
        sys.n = varargin{1};
        sys.m = varargin{2};
        sys.typeflag = varargin{6};
        sys.savedata = [];        
        sys.extra = [];
        %sys.unaltered = 0;
        % Find zero-variables
        constants = find(sys.lmi_variables==0);
        if ~isempty(constants);
            sys.lmi_variables(constants)=[];
            sys.basis(:,1) = sys.basis(:,1) + sum(sys.basis(:,1+constants),2);
            sys.basis(:,1+constants)=[];
        end
        if isempty(sys.lmi_variables)
            sys = full(reshape(sys.basis(:,1),sys.n,sys.m));        
        else
            sys = class(sys,'sdpvar');
        end
        return
    case 7 % Fast version for internal use
        sys.basis = varargin{5};
        sys.lmi_variables=varargin{4};
        sys.n = varargin{1};
        sys.m = varargin{2};
        sys.typeflag = varargin{6};
        sys.savedata = [];
        sys.extra = varargin{7};
        % Find zero-variables
        constants = find(sys.lmi_variables==0);
        if ~isempty(constants);
            sys.lmi_variables(constants)=[];
            sys.basis(:,1) = sys.basis(:,1) + sum(sys.basis(:,1+constants),2);
            sys.basis(:,1+constants)=[];
        end
        if isempty(sys.lmi_variables)
            sys = full(reshape(sys.basis(:,1),sys.n,sys.m));        
        else
            sys = class(sys,'sdpvar');
        end
        return
        
    otherwise
        error('Wrong number of arguments in sdpvar creation');
end

if isempty(n) | isempty(m)
    error('Size must be integer valued')
end;
if ~((abs((n-ceil(n)))+ abs((m-ceil(m))))==0)
    error('Size must be integer valued')
end

lmi_variables = (1:nvar)+varCount;
%lmi_variables = lmi_variables(:);

varCount = lmi_variables(end);

switch matrix_type
    
    case 'full'
        basis = [spalloc(n*m,1,0) speye(nvar)];
        
    case 'full complex'
        basis = [spalloc(n*m,1,0) speye(nvar/2) speye(nvar/2)*sqrt(-1)];
        
    case 'symm'
        basis = spalloc(n^2,1+nvar,2);
        l = 2;
        an_empty = spalloc(n,n,2);
        for i=1:n
            temp = an_empty;
            temp(i,i)=1;
            basis(:,l)=temp(:);
            l = l+1;
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=1;
                temp(j,i)=1;
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        
    case 'symm complex'
        basis = spalloc(n^2,1+nvar,2);
        l = 2;
        an_empty = spalloc(n,n,2);
        for i=1:n
            temp = an_empty;
            temp(i,i)=1;
            basis(:,l)=temp(:);
            l = l+1;
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=1;
                temp(j,i)=1;
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        for i=1:n
            temp = an_empty;
            temp(i,i)=sqrt(-1);
            basis(:,l)=temp(:);
            l = l+1;
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=sqrt(-1);
                temp(j,i)=sqrt(-1);
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        
    case 'herm complex'
        basis = spalloc(n^2,1+nvar,2);
        l = 2;
        an_empty = spalloc(n,n,2);
        for i=1:n
            temp = an_empty;
            temp(i,i)=1;
            basis(:,l)=temp(:);
            l = l+1;
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=1;
                temp(j,i)=1;
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        for i=1:n
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=sqrt(-1);
                temp(j,i)=-sqrt(-1);
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        
    case 'skew'
        basis = spalloc(n^2,1+nvar,2);
        l = 2;
        an_empty = spalloc(n,n,2);
        for i=1:n
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=1;
                temp(j,i)=-1;
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        
    case 'skew complex'
        basis = spalloc(n^2,1+nvar,2);
        l = 2;
        an_empty = spalloc(n,n,2);
        for i=1:n
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=1;
                temp(j,i)=-1;
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        for i=1:n           
            for j=i+1:n,
                temp = an_empty;
                temp(i,j)=sqrt(-1);
                temp(j,i)=-sqrt(-1);
                basis(:,l)=temp(:);
                l = l+1;
            end
        end
        
    case 'toep'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(n,1,1);
        for i=1:n,
            v = an_empty;
            v(i)=1;
            temp = sparse(toeplitz(v));
            basis(:,i+1) = temp(:);
        end
        
        % Notice, complex Toeplitz not Hermitian
    case 'toep complex'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(n,1,1);
        for i=1:n,
            v = an_empty;
            v(i)=1;
            temp = sparse(toeplitz(v));
            basis(:,i+1) = temp(:);
        end
        for i=1:n,
            v = an_empty;
            v(i)=sqrt(-1);
            temp = sparse(toeplitz(v));
            basis(:,n+i+1) = temp(:);
        end
        
    case 'hank'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(n,1,1);
        for i=1:n,
            v = an_empty;
            v(i)=1;
            temp = sparse(hankel(v));
            basis(:,i+1) = temp(:);
        end
        
    case 'hank complex'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(n,1,1);
        for i=1:n,
            v = an_empty;
            v(i)=1;
            temp = sparse(hankel(v));
            basis(:,i+1) = temp(:);
        end
        for i=1:n,
            v = an_empty;
            v(i)=sqrt(-1);
            temp = sparse(hankel(v));
            basis(:,n+i+1) = temp(:);
        end
        
    case 'rhankel'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(2*n-1,1,1);
        for i=1:nvar,
            v = an_empty;
            v(i)=1;
            temp = sparse(hankel(v(1:n),[v(n);v(n+1:2*n-1)]));
            basis(:,i+1) = temp(:);
        end	
        
    case 'rhankel complex'
        basis = spalloc(n^2,1+nvar,2);
        an_empty = spalloc(2*n-1,1,1);
        for i=1:nvar/2,
            v = an_empty;
            v(i)=1;
            temp = sparse(hankel(v(1:n),[v(n);v(n+1:2*n-1)]));
            basis(:,i+1) = temp(:);
        end	
        for i=1:nvar/2,
            v = an_empty;
            v(i)=sqrt(-1);
            temp = sparse(hankel(v(1:n),[v(n);v(n+1:2*n-1)]));
            basis(:,nvar/2+i+1) = temp(:);
        end	
        
    otherwise
        error('Bug! Report')
end

% Update monomtable and pre-calculated variable type
[mt,variabletype] = yalmip('monomtable');
n_mt = size(mt,1);
m_mt = size(mt,2);
if min(lmi_variables)>m_mt % New variables
    [i,j,k] = find(mt);
    mt = sparse([i;lmi_variables(:)],[j;lmi_variables(:)],[k;ones(length(lmi_variables(:)),1)],max(n_mt,max(lmi_variables)),max(m_mt,max(lmi_variables)));
else
    mt(lmi_variables,lmi_variables) = speye(length(lmi_variables));
end
variabletype(size(mt,1),1) = 0;
yalmip('setmonomtable',mt,variabletype);

% Create an object
sys.basis=basis;
sys.lmi_variables = lmi_variables;
sys.n = n;
sys.m = m;
sys.typeflag = 0;
sys.savedata = [];
sys.extra = [];
%sys.unaltered = 1;
sys = class(sys,'sdpvar');
%superiorto('double')
%inferiorto('logdet')

% Typeflags 
% 0 Standard variable
% 1 Inequality (LMI)
% 2 Inequality (element)
% 3 Equality
% 4 Cone
% 5 norm object
% 6 logdet object
% 8 KYP object
