clc
echo on
%*********************************************************
%
% Global bilinear programming
%
%*********************************************************
%
% YALMIP comes with a simple branch-and-bound code
% for solving problems with bilinear/quadratic constraints.
%
% The code is under development, and is currently only 
% applicable to small problems. 
%
% The examples below are hard-coded to use the solvers
% GLPK, PENSDP and PENBMI. Other combinations are possible
% but most experimentation has been performed using this set-up.
%
% It should be possible to replace GLPK and PENSDP with any
% LP or SDP code, but PENBMI is needed for the local problems.
% (not entirely true, you can always try 'none' as local solver)

pause
yalmip('clear')
clc
%*********************************************************
% The first example is the problem from the moment 
% relaxation demo (concave quadratic constraint).
%*********************************************************
x1 = sdpvar(1,1);
x2 = sdpvar(1,1);
x3 = sdpvar(1,1);

objective = -2*x1+x2-x3;

F = set(x1*(4*x1-4*x2+4*x3-20)+x2*(2*x2-2*x3+9)+x3*(2*x3-13)+24>0);
F = F + set(4-(x1+x2+x3)>0);
F = F + set(6-(3*x2+x3)>0);
F = F + set(x1>0);
F = F + set(2-x1>0);
F = F + set(x2>0);
F = F + set(x3>0);
F = F + set(3-x3>0);
pause

% Explicitly specify solvers (this is currently recommended)
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','glpk');   % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
pause

% Feasible solutions and upper bounds are obtained using PENBMI
% The following tuning is fairly stable for the problems in this example.
ops = sdpsettings(ops,'penbmi.P0',0.01);     
pause

% Solve!
solvesdp(F,objective,ops)
pause
yalmip('clear')
clc
%***********************************************************
% The second example is slightly larger. It is a problem 
% with a concave quadratic objective function, and two 
% concave quadratic constraints.
%***********************************************************
x1 = sdpvar(1,1);
x2 = sdpvar(1,1);
x3 = sdpvar(1,1);
x4 = sdpvar(1,1);
x5 = sdpvar(1,1);
x6 = sdpvar(1,1);

objective = -25*(x1-2)^2-(x2-2)^2-(x3-1)^2-(x4-4)^2-(x5-1)^2-(x6-4)^2;

F = set((x3-3)^2+x4>4) + set((x5-3)^2+x6>4);
F = F + set(x1-3*x2<2) + set(-x1+x2<2) + set(x1+x2>2);
F = F + set(6>x1+x2>2);
F = F + set(1<x3<5) + set(0<x4<6)+set(1<x5<5)+set(0<x6<10)+set(x1>0)+set(x2>0);

pause

% The global solver in YALMIP can currently only deal with linear objectives.
% A simple epi-graph formulation solves this.
t = sdpvar(1,1);
F = F + set(objective<t);
pause

% Solve!
solvesdp(F,t,ops);
pause
clc
%***********************************************************
% Random bound-constrained indefinte quadratic program.
%***********************************************************

n = 5;
x = sdpvar(n,1);
t = sdpvar(1,1);

A = randn(n,n);A = A*A';
B = randn(n,n);B = B*B';

objective = x'*A*x-x'*B*x;

F = set(-1 < x < 1) + set(objective<t);
pause

% Solve!
solvesdp(F,t,ops);
pause
yalmip('clear')
clc
%***********************************************************
% Random boolean quadratic programming.
%
% By adding a nonlinear constraint x(x-1)==0, we can solve
% boolean programming using global nonlinear programming.
%***********************************************************

n = 10;
x = sdpvar(n,1);
t = sdpvar(1,1);

Q = randn(n,n);Q = Q*Q'/norm(Q)^2;
c = randn(n,1);

objective = x'*Q*x+c'*x;

F = set(x.*(x-1)==0) + set(objective < t);
pause

% Note, all complicating variables (variables involved in nonlinear terms)
% must be bounded (either explicitely or implicetely via linear and relaxed 
% nonlinear constraints).
%
% The easible set for the relaxed problem is not bounded
% so YALMIP will not be able to derive variable bounds.
% We add some valid bounds to fix this
F = F + set(0<x<1);
pause

% Solve!
solvesdp(F,t,ops);
double(x'*Q*x+c'*x)
pause

% Let us compare with an integer solver (here the internal solver)
solvesdp(set(binary(x)),x'*Q*x+c'*x,sdpsettings('solver','bnb'));
double(x'*Q*x+c'*x)
pause

yalmip('clear')
clc
%***********************************************************
% This example is a somewhat larger indefinite quadratic
% programming problem, taken from the GAMS problem library.
%***********************************************************
pause

% The problem has 11 variables
x1 = sdpvar(1,1);
x2 = sdpvar(1,1);
x3 = sdpvar(1,1);
x4 = sdpvar(1,1);
x5 = sdpvar(1,1);
x6 = sdpvar(1,1);
x7 = sdpvar(1,1);
x8 = sdpvar(1,1);
x9 = sdpvar(1,1);
x10 = sdpvar(1,1);
x11 = sdpvar(1,1);
t = sdpvar(1,1);
x = [x1;x2;x3;x4;x5;x6;x7;x8;x9;x10;x11];

pause

% ...and 22 linear constraints
e1 =  - x1 - 2*x2 - 3*x3 - 4*x4 - 5*x5 - 6*x6 - 7*x7 - 8*x8 - 9*x9 - 10*x10      - 11*x11 	- 0;
e2 =   - 2*x1 - 3*x2 - 4*x3 - 5*x4 - 6*x5 - 7*x6 - 8*x7 - 9*x8 - 10*x9 - 11*x10      - x11 	- 0;
e3 =   - 3*x1 - 4*x2 - 5*x3 - 6*x4 - 7*x5 - 8*x6 - 9*x7 - 10*x8 - 11*x9 - x10      - 2*x11 	- 0;
e4 =   - 4*x1 - 5*x2 - 6*x3 - 7*x4 - 8*x5 - 9*x6 - 10*x7 - 11*x8 - x9 - 2*x10      - 3*x11 	- 0;
e5 =   - 5*x1 - 6*x2 - 7*x3 - 8*x4 - 9*x5 - 10*x6 - 11*x7 - x8 - 2*x9 - 3*x10      - 4*x11 	- 0;
e6 =   - 6*x1 - 7*x2 - 8*x3 - 9*x4 - 10*x5 - 11*x6 - x7 - 2*x8 - 3*x9 - 4*x10      - 5*x11 	- 0;
e7 =   - 7*x1 - 8*x2 - 9*x3 - 10*x4 - 11*x5 - x6 - 2*x7 - 3*x8 - 4*x9 - 5*x10      - 6*x11 	- 0;
e8 =   - 8*x1 - 9*x2 - 10*x3 - 11*x4 - x5 - 2*x6 - 3*x7 - 4*x8 - 5*x9 - 6*x10      - 7*x11 	- 0;
e9 =   - 9*x1 - 10*x2 - 11*x3 - x4 - 2*x5 - 3*x6 - 4*x7 - 5*x8 - 6*x9 - 7*x10      - 8*x11 	- 0;
e10 =   - 10*x1 - 11*x2 - x3 - 2*x4 - 3*x5 - 4*x6 - 5*x7 - 6*x8 - 7*x9 - 8*x10       - 9*x11 	- 0;
e11 =   - 11*x1 - x2 - 2*x3 - 3*x4 - 4*x5 - 5*x6 - 6*x7 - 7*x8 - 8*x9 - 9*x10       - 10*x11 	- 0;
e12 =     x1 + 2*x2 + 3*x3 + 4*x4 + 5*x5 + 6*x6 + 7*x7 + 8*x8 + 9*x9 + 10*x10       + 11*x11 	- 66;
e13 =     2*x1 + 3*x2 + 4*x3 + 5*x4 + 6*x5 + 7*x6 + 8*x7 + 9*x8 + 10*x9 + 11*x10       + x11 	- 66;
e14 =     3*x1 + 4*x2 + 5*x3 + 6*x4 + 7*x5 + 8*x6 + 9*x7 + 10*x8 + 11*x9 + x10       + 2*x11 	- 66;
e15 =     4*x1 + 5*x2 + 6*x3 + 7*x4 + 8*x5 + 9*x6 + 10*x7 + 11*x8 + x9 + 2*x10       + 3*x11 	- 66;
e16 =     5*x1 + 6*x2 + 7*x3 + 8*x4 + 9*x5 + 10*x6 + 11*x7 + x8 + 2*x9 + 3*x10       + 4*x11 	- 66;
e17 =     6*x1 + 7*x2 + 8*x3 + 9*x4 + 10*x5 + 11*x6 + x7 + 2*x8 + 3*x9 + 4*x10       + 5*x11 	- 66;
e18 =     7*x1 + 8*x2 + 9*x3 + 10*x4 + 11*x5 + x6 + 2*x7 + 3*x8 + 4*x9 + 5*x10       + 6*x11 	- 66;
e19 =     8*x1 + 9*x2 + 10*x3 + 11*x4 + x5 + 2*x6 + 3*x7 + 4*x8 + 5*x9 + 6*x10       + 7*x11 	- 66;
e20 =     9*x1 + 10*x2 + 11*x3 + x4 + 2*x5 + 3*x6 + 4*x7 + 5*x8 + 6*x9 + 7*x10       + 8*x11 	- 66;
e21 =     10*x1 + 11*x2 + x3 + 2*x4 + 3*x5 + 4*x6 + 5*x7 + 6*x8 + 7*x9 + 8*x10       + 9*x11 	- 66;
e22 =     11*x1 + x2 + 2*x3 + 3*x4 + 4*x5 + 5*x6 + 6*x7 + 7*x8 + 8*x9 + 9*x10       + 10*x11 	- 66;

pause


% Define the objective function and the whole program
obj =  (0.5*x1*x2 - x1*x1 + 0.5*x2*x1 - x2*x2 + 0.5*x2*x3 + 0.5*x3*x2 - x3*x3       + 0.5*x3*x4 + 0.5*x4*x3 - x4*x4 + 0.5*x4*x5 + 0.5*x5*x4 - x5*x5 + 0.5*x5      *x6 + 0.5*x6*x5 - x6*x6 + 0.5*x6*x7 + 0.5*x7*x6 - x7*x7 + 0.5*x7*x8 + 0.5      *x8*x7 - x8*x8 + 0.5*x8*x9 + 0.5*x9*x8 - x9*x9 + 0.5*x9*x10 + 0.5*x10*x9       - x10*x10 + 0.5*x10*x11 + 0.5*x11*x10 - x11*x11);
e = -[e1;e2;e3;e4;e5;e6;e7;e8;e9;e10;e11;e12;e13;e14;e15;e16;e17;e18;e19;e20;e21;e22];
F = set(10>x>0);
F = F + set(e>0);
F = F + set(obj==t);

% We will nowe try to solve this using the same strategy as before
solvesdp(F,t,ops)

pause
clc

% As an alternative, we might use additional cuts using the CUT functionality.
% These additional constraints are only used in domain reduction and
% solution of lower bound programs, but not in the local solver.

% We square all linear constraints and add these as cuts
f = sdpvar(F(find(is(F,'linear'))));
X = triu(f*f');
ind = find(triu(ones(length(X))));
F = F + cut(X(ind) > 0);
pause

% ...and solve (To speed up things, we turn off LP based domain reduction)
solvesdp(F,t,sdpsettings(ops,'bmibnb.lpreduce',0))
pause
clc

% The global solver in YALMIP can only solve bilinear programs, 
% but a pre-processor is capable of transforming higher order problems 
% to bilinear programs. As an example, the variable x^3y^2 will be replaced 
% with the the variable w and the constraints w == uv, u == zx, z == x^2, v == y^2. 
% The is done automatically if the global solver, or PENBMI, is called with 
% a higher order polynomial problem. Note that this conversion is rather 
% inefficient, and only very small problems can be addressed using this simple approach.
% Additionally, this module has not been tested in any serious sense.
pause

% Let us solve a small trivial polynomial program
sdpvar x y
F = set(x^3+y^5 < 5) + set(y > 0);
F = F + set(-100 < [x y] < 100) % Always bounded domain
pause

solvesdp(F,-x,ops)
pause

%***********************************************************
% The main motivation for implementing a bilinear solver in
% YALMIP is matrix-valued problems, i.e. BMI problems.
%
% Of-course, BMI problems are even harder than the nonconvex 
% quadratic problems solved above. However, in some cases, 
% it is actually possible to solve BMI problems globally.
%
% Don't expect too much though...
%***********************************************************

pause
clc

%***********************************************************
% The following is a classical problem
%***********************************************************
A0 = [-10 -0.5 -2;-0.5 4.5 0;-2 0 0];
A1 = [9 0.5 0;0.5 0 -3 ; 0 -3 -1];
A2 = [-1.8 -0.1 -0.4;-0.1 1.2 -1;-0.4 -1 0];
K12 = [0 0 2;0 -5.5 3;2 3 0];

x = sdpvar(1,1);
y = sdpvar(1,1);
t = sdpvar(1,1);

F = set(A0+x*A1+y*A2+x*y*K12-t*eye(3)<0);
F = F + set(-0.5 < x < 2) + set(-3 < y < 7);
pause

% Specify solvers
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','pensdp'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
ops = sdpsettings(ops,'penbmi.P0',0.01);              % Typically good

pause

% Solve!
solvesdp(F,t,ops);
pause
clc
%***********************************************************
% The next example shows how to solve a standard LQ control
% problem with constraints on the feedback gain
%***********************************************************

A = [-1 2;-3 -4];B = [1;1];

P = sdpvar(2,2);
K = sdpvar(1,2);

F = set(P>0)+set((A+B*K)'*P+P*(A+B*K) < -eye(2)-K'*K);
F = F + set(K<0.1)+set(K>-0.1);
F = F + set(1000> P(:)>-1000);
solvesdp(F,trace(P),ops);
pause
clc
%***********************************************************
% Decay-rate estimation as a global BMI example ...
%
% Note, this is a quasi-convex problem that PENBMI actually 
% solves to global optimality, so the use of a global solver
% is pretty stupid. It is only included here to show some 
% properties of the global solver.
%
% (will run for 100 iterations)
%***********************************************************

A = [-1 2;-3 -4];
t = sdpvar(1,1);
P = sdpvar(2,2);
F = set(P>eye(2))+set(A'*P+P*A < -2*t*P);
F = F + set(-1e4 < P(:) < 1e4) + set(100 > t > 0) ;
pause
solvesdp(F,-t,ops);
pause

% Now, that sucked, right! 100 iterations and still 10% gap...
%
% The way we model a problem can make a huge difference.
% The decay-rate problem can be substantially improved by 
% using a smarter model. The original problem is homogenous 
% w.r.t P, and a better way to make the feasible set bounded
% is to constrain the trace of P
A = [-1 2;-3 -4];
t = sdpvar(1,1);
P = sdpvar(2,2);
F = set(P>0)+set(A'*P+P*A < -2*t*P);
F = F + set(trace(P)==1) + set(100 > t > 0) ;
pause
solvesdp(F,-t,ops);
pause

% Nothing prevents us from adding an additional 
% constraint derived from trace(P)=1
F = F + set(trace(A'*P+P*A) < -2*t);
pause

solvesdp(F,-t,ops);
pause

% When we add redundant constraints, we improve the relaxations
% and obtain better lower bounds. For the upper bounds however,
% these cuts only add additional computational burden.
% To overcome this, the command CUT can be used.
% Constraints defined with CUT are not used when the upper 
% bound problems are solved, but are only used in relaxations
F = set(P>0)+set(A'*P+P*A < -2*t*P);
F = F + set(trace(P)==1) + set(100 > t > 0) ;
F = F + cut(trace(A'*P+P*A) < -2*t);
pause

solvesdp(F,-t,ops);
pause

% Finally, it should be mentioned that the branch & bound
% algorithm can run without any installed local BMI solver. 
% This version of the algorithms is obtained by specifying 
% the upper bound solver as 'none'
ops = sdpsettings(ops,'bmibnb.uppersolver','none');
F = set(P>0)+set(A'*P+P*A < -2*t*P);
F = F + set(trace(P)==1) + set(100 > t > 0) ;
F = F + cut(trace(A'*P+P*A) < -2*t);
pause

solvesdp(F,-t,ops);
pause

echo off



break


% Using cuts instead
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','pensdp'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
ops = sdpsettings(ops,'penbmi.P0',0.01);              % Typically good

A = [-1 2;-3 -4];
t = sdpvar(1,1);
P = sdpvar(2,2);
F = set(P>0)+set(A'*P+P*A < -2*t*P);
F = F + set(trace(P)==1) + set(100 > t > 0) ;
F = F + cut(trace(A'*P+P*A) < -2*t);
F = F + cut([-A'*P-P*A P*t;P*t P*t/2]>0)
solvesdp(F,-t,ops);



clear all

ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','pensdp'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
ops = sdpsettings(ops,'bmibnb.gaptol',1e-3);      % LP solver

n = 2;
m = 2;
k = 8;
x = sdpvar(n,1);
y = sdpvar(m,1);
lambda = sdpvar(1,1);
B = -100+200*rand(k,k);
for i = 1:n
    B = B + x(i)*(-100+200*rand(k,k));    
end
for j = 1:n
    B = B + y(j)*(-100+200*rand(k,k));    
end

for i = 1:n
    for j = 1:m
         B = B + x(i)*y(j)*(-100+200*rand(k,k));    
    end
end
B = (B+B')/2;

F = set(lambda*eye(k)-B>0) + set(-50<[x;y]<50);% + set(-10000 < lambda <10000);

solvesdp(F,lambda,ops);


ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','pensdp'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','nag');      % LP solver
ops = sdpsettings(ops,'bmibnb.gaptol',1e-3);      % LP solver

solvesdp(set(1>x>0),x(1)^2+x(3)^2-0.1*x(2)^2,ops)

yalmip('clear')
x = sdpvar(1,1);
t = -1:0.1:2;
plot(t,t.^2)
l = -1;
u = 2;
x = sdpvar(1,1);
F = set([(x-l)*(x-l);(u-x)*(x-l);(u-x)*(u-x);x-l;u-x])
plot(F,[x;x^2],'r',25,sdpsettings('relax',1,'solver','glpk','convertconvexquad',0,'verbose',0))
v = [(x-l)*(x-l);(u-x)*(x-l);(u-x)*(u-x);x-l;u-x]



solvesdp(set(t>(x-0.5)'*Q*(x-0.5)+sum(x))+set(x.*(x-1)==0)+set(1>x>0),t,ops)


yalmip('clear')
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','glpk'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver

n = 5;
x = sdpvar(n,1);
t = sdpvar(1,1);x'*(A-B)*x
z = sdpvar(n,1);
%A = randn(n,n);A = A*A';
%B = randn(n,n);B = B*B';
%z = sdpvar(5,1);
[v,d] = eig(A-B);
%F = set(-1 < x < 1) + set(x'*(A-B)*x==t) ;
%F = set(-1 < x < 1) + set(z'*d*z == t) + set(v'*x==z);
%F = set(-1 < (v'\z) < 1) + set(z'*d*z == t);
%F = set(-1 < x < 1) + set(z'*d*z == t) + set(v'*x==z) + set(x'*(A-B)*x==t) ;
F = set(-1 < x < 1) + set(z'*d*z == t) + set(v'*x==z) + set(x'*(A-B)*x==t)+set(-1e3<[x;z;t]<1e3);

solvesdp(F,t)



zz = sdpvar(n,1);
xx = sdpvar(n,1);
tt = sdpvar(1,1);
[v,d] = eig(A-B);
Ff = set(-1 < xx < 1) + set(zz'*d*zz == tt) + set(v'*xx==zz) + set(xx'*(A-B)*xx==tt)+set(-1e3<[xx;zz;tt]<1e3)

solvesdp(Ff,tt)











n = 5;
t = sdpvar(1,1);
z = sdpvar(n,1);z'*z
x = sdpvar(n,1);
x'*(A-B)*x;
[v,d] = eig(A-B);
F = set(-1 < x < 1) + set(z'*d*z < t) + set(v'*x==z) + set(x'*(A-B)*x<t)+set(-1e3<[x;z;t]<1e3);
solvesdp(F,t);

tt = sdpvar(1,1);
xx = sdpvar(n,1);
zz = sdpvar(n,1);
[v,d] = eig(A-B);
Ff = set(-1 < xx < 1) + set(zz'*d*zz < tt) + set(v'*xx==zz) + set(xx'*(A-B)*xx<tt)+set(-1e3<[xx;zz;tt]<1e3);
solvesdp(Ff,tt);




yalmip('clear')
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','glpk'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
n = 5;
t = sdpvar(1,1);
x = sdpvar(n,1);
A = randn(n,n);A = A*A';
[v,d] = eig(A);
F = set(-1 < (v'\x)+2 < 1) + set(x'*d*x==t);
solvesdp(F,t,ops);
F = set(-1 < x < 1) + set((x-2)'*A*(x-2)==t);
solvesdp(F,t,ops);





n = 1;A = 2;B = 3;
t = sdpvar(1,1);
x = sdpvar(n,1);
x'*(A-B)*x;
z = sdpvar(n,1);
[v,d] = eig(A-B);
F = set(-1 < x < 1) + set(z'*d*z == t) + set(v'*x==z) + set(x'*(A-B)*x==t)+set(-1e3<[x;z;t]<1e3);
solvesdp(F,t,sdpsettings('solver','penbmi'));

tt = sdpvar(1,1);A = 2;B = 3;
xx = sdpvar(n,1);
zz = sdpvar(n,1);
[v,d] = eig(A-B);
Ff = set(-1 < xx < 1) + set(zz'*d*zz == tt) + set(v'*xx==zz) + set(xx'*(A-B)*xx==tt)+set(-1e3<[xx;zz;tt]<1e3);
solvesdp(Ff,tt,sdpsettings('solver','penbmi'));



ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','pensdp'); % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver


A = [-1 2 0;-3 -4 0;0 0 1];B = [1;1;1];

P = sdpvar(3,3);
K = sdpvar(1,3);

%F = set(P>0)+set((A+B*K)'*P+P*(A+B*K) < -eye(3)-K'*K);
F = set(P>0)+set([-(A+B*K)'*P-P*(A+B*K) -eye(3) K';K 1]);

F = F + set(K<3)+set(K>-3);
F = F + set(1000> P(:)>-1000);
solvesdp(F,trace(P),ops);




yalmip('clear')
x1 = sdpvar(1,1);
x2 = sdpvar(1,1);
x3 = sdpvar(1,1);
x4 = sdpvar(1,1);
x5 = sdpvar(1,1);
x = [x1;x2;x3;x4;x5];

y1 = sdpvar(1,1);
y2 = sdpvar(1,1);
y3 = sdpvar(1,1);
y4 = sdpvar(1,1);
y5 = sdpvar(1,1);
y = [y1;y2;y3;y4;y5];

obj = 0.5*((x2+x3-4*x1)*y1+(3*x1-2*x3+x5)*y2+(3*x1-2*x2+x4)*y3+(x3-2*x1)*y4+(x2-2*x1)*y5)+x1;

F = set([]);
F = F + set( (x1-x2)^2 + (y1-y2)^2 < 1);
F = F + set( (-x1+x3-x5)^2 + (y1-y3+y5)^2 < 1);
F = F + set( (x1-x2+x4)^2 + (y1-y2+y4)^2 < 1);
F = F + set( (-x1+x3)^2 + (y1-y3)^2 < 1);
F = F + set( (2*x1-x2-x3+x5)^2 + (-y2+y3-y5)^2 < 1);
F = F + set( (2*x1-x2)^2 + (y2)^2 < 1);
F = F + set( (x1-x2)^2 + (y1-y2-1)^2 < 1);
F = F + set( (2*x1-x2-x3)^2 + (-y2+y3)^2 < 1);
F = F + set( (x3-x5)^2 + (-y3+y5)^2 < 1);
F = F + set( (-x1+x3-x5)^2 + (y1-y3+y5-1)^2 < 1);
F = F + set( (2*x1+x3-x5)^2 + (-y3+y5)^2 < 1);
F = F + set( (2*x1-x2-x3+x4+x5)^2 + (-y2+y3+y4-y5)^2 == 1);
F = F + set( (-2*x1+x2-x4)^2 + (y2-y4)^2 < 1);
F = F + set( (x1-x2+x4)^2 + (y1-y2+y4-1)^2 < 1);
F = F + set( (x1-x3)^2 + (1-y1+y3)^2 < 1);
F = F + set( (x2-x4)^2 + (y2-y4)^2 < 1);
F = F + set( (2*x1-x3)^2 + (y3)^2 < 1);
F = F + set( (2*x1-x2-x3+x4)^2 + (-y2+y3+y4)^2 < 1);
F = F + set(x2-x3>0)
F = F + set(x.^2+y.^2 == 1);
F = F + set(0<x1<0.5);
F = F + set(0<x(2:end)<1);
F = F + set(-1<y<1);
t = sdpvar(1,1);
F = F + set(obj>t)+set(1>t>0.55);
ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','glpk');   % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk'); % Local solver
ops = sdpsettings(ops,'bmibnb.roottight',1);      % LP solver
ops = sdpsettings(ops,'bmibnb.lpreduce',0);      % LP solver
ops = sdpsettings(ops,'penbmi.P0',0.09);      % LP solver
ops = sdpsettings(ops,'bmibnb.maxiter',150);      % LP solver
ops = sdpsettings(ops,'bmibnb.lowrank',1);      % LP solver
solvesdp(F,-t,ops);

G = lmi;
p = obj;
lambdas = [];
for i = 1:length(F)
    if is(F(i),'equality')
        lambda = sdpvar(length(sdpvar(F(i))),1);
        p = p+lambda'*sdpvar(F(i));
    else
        lambda = sdpvar(length(sdpvar(F(i))),1);
                G = G + set(lambda>0);
        p = p-lambda'*sdpvar(F(i)); 
    end
    lambdas = [lambdas;lambda];
end
t = sdpvar(1,1);
p = p-t;
solvesos(p,[],[t;lambdas],-t,G);

Z =double([0 0;x1-x2 y1-y2;x3-x1-x5 y1-y3+y5;-x1 y1;0 1;x1 y1;x1-x2+x4 y1-y2+y4;x3-x1 y1-y3]);
Zopt = [0 0;-0.40980 0.22442;-0.5 0.63947;-0.26214 0.96503;0 1;0.26214 0.96503;0.5 0.63947;0.40980 0.22442]

t = (0:1:100)*2*pi/100;
patch(0.5*cos(t),0.5+0.5*sin(t),'b');
hold on
patch(Zopt(:,1),Zopt(:,2),'r')

% Structural design from COCOS
x1 = sdpvar(1,1);
x2 = sdpvar(1,1);
y1 = sdpvar(1,1);
y2 = sdpvar(1,1);
t = sdpvar(1,1);
obj = 300*y1+300*y2+(x1+x2)^2+x2^2;

F = set([x1;x2]>0)+set(x1*y2==10)+set(x2*y2==20)+set((y1-1)*(y1-2)==0)+set((y2-1)*(y2-2)==0);
F = F + set(2>[y1;y2]>1);
F = F + set(200>[x1;x2]>-200);
F = F + set(t==obj);

ops = sdpsettings('solver','bmibnb');                 % Global solver
ops = sdpsettings(ops,'bmibnb.lowersolver','glpk');   % Lower solver
ops = sdpsettings(ops,'bmibnb.uppersolver','penbmi'); % Local solver
ops = sdpsettings(ops,'bmibnb.lpsolver','glpk');      % LP solver
ops = sdpsettings(ops,'bmibnb.maxiter',15);      % LP solver
ops = sdpsettings(ops,'bmibnb.roottight',1);      % LP solver
ops = sdpsettings(ops,'bmibnb.lpreduce',1);      % LP solver
solvesdp(F,t,ops)



A = [-1 2;3 4];B = [1;1];
P = sdpvar(2,2);
Y = sdpvar(2,2);
K = sdpvar(1,2);
F = set(P>0)+set((A+B*K)'*P+P*(A+B*K) < -eye(2));
F = F + set([-Y*(A+B*K)-(A+B*K)*Y Y;Y eye(2)])
F = F + set(K<8)+set(K>-8);
F = F + set(100> P(:)>-100);
F = F + set(100> Y(:)>-100);
F = F + cut([P eye(2);eye(2) Y]);
F = F + cut(P*Y == eye(2));
solvesdp(F,trace(P),sdpsettings(ops,'bmibnb.lowersolver','sedumi'));



A = [-1 2;-3 -4];B = [1;1];

P = sdpvar(2,2);
K = sdpvar(1,1);
C = [1 1];
F = set(P>0)+set((A+B*K*C)'*P+P*(A+B*K*C) < -eye(2)-K'*K);
F = F + set(K<8)+set(K>-8);
F = F + set(100> P(:)>-100)+set(P>0.85*pp);
solvesdp(F,trace(P),sdpsettings(ops,'bmibnb.lowersolver','sedumi'));


A = [2 2;3 4];
B = [1;1];
C = [1 1];
P = sdpvar(2,2);
Y = sdpvar(2,2);
K = sdpvar(1,2);
%K = [-2 -3];
F = set([Y-eye(2)-K'*K Y*(A+B*K)';(A+B*K)*Y Y])+set(-100<Y(:)<100)+set(-10<K<10);
%F = set([Y-eye(2) Y*(A+B*K)' Y*K';(A+B*K)*Y Y zeros(2,1);K*Y zeros(1,2) eye(1)])+set(-100<Y(:)<100)+set(-10<K<10);
solvesdp(F,-trace(Y),sdpsettings(ops,'bmibnb.lowersolver','sedumi'));










