function [Fdual,objdual,X,t,err] = dualize(F,obj)
% DUALIZE Create the dual of a SDP given in primal form
%
% [Fd,objd,X,t,err] = dualize(F,obj)
%
% Input
%  F   : Primal constraint in form AX=b+dt, X>0, t free
%  obj : Primal cost CX+ct
%
% Output
%  Fd  : Dual constraints in form Cj-sum(Aijyi)>0, c-dy==0
%  obj : Dual cost b'y (to be maximized)
%  X   : The detected primal cone variables
%  t   : The detected primal free variables
%  err : Error status (returns 0 if no problems)
%
% Example
%  See the HTML help.
%
% See also DUAL, SOLVESDP, SDPVAR, PRIMALIZE

% Author Johan Lfberg
% $Id: dualize.m,v 1.7 2005/01/05 21:47:37 johanl Exp $

err = 0;

% It's general, but not insanely general...
if ~(islinear(F) & islinear(obj))
    if nargout == 6
        Fdual = set([]);objdual = [];y = []; X = []; t = []; err = 1;
    else
        error('Can only dualize linear problems');
    end
end
if any(is(F,'socc'))
    if nargout == 6
        Fdual = set([]);objdual = [];y = []; X = []; t = []; err = 1;
    else
        error('Cannot dualize second order cone constraints');
    end
end
if any(is(F,'integer')) | any(is(F,'binary'))
    if nargout == 6
        Fdual = set([]);objdual = [];y = []; X = []; t = []; err = 1;
    else
        error('Cannot dualize discrete problems');
    end
end

% Cones with SDP cones and equalities
F_X = set([]);
F_AXb = set([]);

% First, get variables in initial SDP cones
% We need this to avoid adding the same variable twice
% when we add simple LP constraints (as in P>0, P(1,3)>0)
varSDP = [];
Xset = zeros(length(F),1);
for i = 1:length(F)
    if is(F(i),'sdp')
        Fi = sdpvar(F(i));
        if is(Fi,'sdpcone') & length(Fi)>1
            Xset(i) = 1;
            vars = getvariables(Fi);
            varSDP = [varSDP;vars(1) vars(end)];
        end
    end
end
F_X = F(find(Xset));
F = F(find(~Xset));

% Find all LP constraints, add slacks and extract simple cones
elementwise = is(F,'element-wise');
if ~isempty(find(elementwise))
    Flp = F(find(elementwise));
    F = F(find(~elementwise)); % remove these LPs
    % Find LP cones
    lpconstraint = [];
    for i = 1:length(Flp)
        temp = sdpvar(Flp(i));
        lpconstraint = [lpconstraint;temp(:)];
    end
    base = getbase(lpconstraint);
    lpcones = (base(:,1)==0) & (sum(base,2)==1) & (sum(base==1,2)==1);   
    if 1   
            if ~isempty(lpcones)
                x  = lpconstraint(find(lpcones));
                nlp = lpconstraint(find(~lpcones));
                if ~isempty(nlp)
                    s = sdpvar(length(nlp),1);
                    F_AXb = F_AXb + set(nlp-s==0);
                    x = [x;s];
                end
            else
                x = sdpvar(length(lpconstraint),1); % New LP cones
                F_AXb = F_AXb + set(lpconstraint-x==0);
            end
    else
%         effectivelylpcones = (base(:,1)<0) & (sum(base(:,2:end),2)==1) & (sum(base(:,2:end)==1,2)==1);
%         if ~isempty(effectivelylpcones)
%             x = lpconstraint(find(lpcones));
%             xe = lpconstraint(find(effectivelylpcones & ~lpcones));
%             nlp = lpconstraint(find(~effectivelylpcones));
%             if ~isempty(nlp)
%                 s = sdpvar(length(nlp),1);
%                 F_AXb = F_AXb + set(nlp-s==0);
%                 x = [x;s];
%             end
%             if ~isempty(xe)
%                 s = sdpvar(length(xe),1);
%                 F_AXb = F_AXb + set(xe-s==0);
%                 x = [x;s;recover(getvariables(xe))];
%             end
% 
%         else
%             x = sdpvar(length(lpconstraint),1); % New LP cones
%             F_AXb = F_AXb + set(lpconstraint-x==0);
%         end
    end
    
    
    
    xv = getvariables(x);
    x = recover(xv);
    if isempty(varSDP)
    else
        keep = zeros(length(x),1);
        for i = 1:length(xv)            
            if any(varSDP(:,1)<=xv(i) & xv(i) <=varSDP(:,2))
            else
                keep(i) = 1;
            end
        end
        if all(keep)
        else
            xsdp = x(find(~keep))
            s = sdpvar(length(xsdp),1);
            F_AXb = F_AXb + set(xsdp-s==0);
            x = x(find(keep));
            x = [x;s];
        end
    end
else
    x = [];
end

% Check for mixed cones, ie terms C-A'y > 0.
keep = ones(length(F),1);
for i = 1:length(F)
    if is(F(i),'lmi')
        % Semidefinite dual-form cone
        Fi = sdpvar(F(i));
        n = size(Fi,1);
        S = sdpvar(n,n);
        slack = Fi-S;
        ind = find(triu(reshape(1:n^2,n,n)));        
        F_AXb =  F_AXb + set(slack(ind)==0);
        F_X = F_X + set(S);
        keep(i)=0;
    end
end
% Now, remove all mixed cones...
F = F(find(keep));

% Get the equalities
AXbset = is(F,'equality');
if any(AXbset)
    % Get the constraints
    F_AXb = F_AXb + F(find(AXbset));
    F = F(find(~AXbset));
end

% Is thee something we missed in our tests?
if length(F)>0
    error('DUALIZE can only treat standard SDPs (and LPs) at the moment.')    
end

% Free variables (here called t) is everything except the cone variables
all_variables = uniquestripped([depends(obj) depends(F_AXb) depends(F_X) depends(x)]);
t_variables = setdiff(setdiff(all_variables,depends(F_X)),depends(x));
t = recover(t_variables);

% Sort the SDP cone variables X according to YALMIP
% This is just to simplify some indexing later
ns = [];
first_var = [];
for i = 1:length(F_X)
    Xt = sdpvar(F_X(i));
    ns = [ns length(Xt)];
    first_var = [first_var min(getvariables(Xt))];
end
[sorted,index] = sort(first_var);
if ~isempty(t)
    sumAll = sum(t);
else
    sumAll = 0;
end
k = 1;
X = cell(0);
for i = 1:length(F_X)
    XX = sdpvar(F_X(index(i)));
    X{k} = XX;
    ns(k) = length(X{i});
    sumAll = sumAll + sumsum(XX);
    k = k + 1;
end
% Extract vectorized data matrices
vecF = [];
for i = 1:length(F_AXb)
    AXb = sdpvar(F_AXb(i));
    vecF = [vecF;AXb(:)];
end
if isempty(obj)
    obj = 0;
end
if ~isempty(x)
    sumAll = sumAll + sum(x);    
end

vecF = [vecF;obj;sumAll]; % Just to get indicies in YALMIP simple


% Very inefficient treatment of matrix data, but simple and safe...
% Partition into
% F_X : SDP cones
% F_x : LP cones
% F_t : Free cones

% Put free variables last
Fbase = getbase(vecF);

b = Fbase(1:end-2,1);
Fbase = -Fbase(1:end-1,2:end);
vecA = [];
x_variables = getvariables(x);
ind = ismembc(all_variables,t_variables);
t_in_all = find(ind);
ind = ismembc(all_variables,x_variables);
x_in_all = find(ind);
Fbase_t = Fbase(:,t_in_all);
Fbase_x = Fbase(:,x_in_all);
Fbase_X = Fbase;
Fbase_X(:,unionstripped(t_in_all,x_in_all)) = [];

start = 0;
n_cones = length(ns);
% All LPs in one cone
if length(x)>0
    n_lp = 1;
else
    n_lp = 0;
end
n_free = length(t_variables);

% SDP cones
for j = 1:1:n_cones

    ind = reshape(1:ns(j)^2,ns(j),ns(j));
    ind = find(tril(ind));

    % Get non-symmetric constraint AiX=b
    Fi = Fbase_X(1:length(b),start+(1:length(ind)))'/2;
    Ai = spalloc(ns(j)^2,length(b),nnz(Fi));
    Ai(ind,:) = Fi;
    % Symmetrize
    [rowcols,varindicies,vals]=find(Ai);
    the_col = 1+floor((rowcols-1)/ns(j));
    the_row = rowcols-(the_col-1)*ns(j);
    these_must_be_transposed = find(the_row > the_col);
    new_rowcols = the_col(these_must_be_transposed) + (the_row(these_must_be_transposed)-1)*ns(j);
    Ai(sub2ind(size(Ai),new_rowcols,varindicies(these_must_be_transposed))) = vals(these_must_be_transposed);
    % Fix diagonal term
    diags = find(diag(1:ns(j)));
    Ai(diags,:) = 2*Ai(diags,:);
    vecA{j} = Ai;
    start = start + length(ind);
end
% LP Cone
for j = 1:1:n_lp
    n = length(x);
    % Get non-symmetric constraint AiX=b
    Ai = Fbase_x(1:length(b),:)';
    vecA{j+n_cones} = Ai;
    start = start + n;
end
% FREE VARIABLES
start = 0;
for j = 1:1:n_free
    ind = 1;
    % Get non-symmetric constraint AiX=b
    Fi = Fbase_t(1:length(b),start+1)';
    Ai = spalloc(1,length(b),nnz(Fi));
    Ai(ind,:) = Fi;
    % Fix diagonal term
    vecA{j+n_cones+n_lp} = Ai;
    start = start + 1;
end

% COST MATRIX
% SDP CONE
start = 0;
for j = 1:1:n_cones
    ind = reshape(1:ns(j)^2,ns(j),ns(j));
    ind = find(tril(ind));
    C{j} = spalloc(ns(j),ns(j),0);
    C{j}(ind) = -Fbase_X(end,start+(1:length(ind)));
    C{j} = (C{j}+ C{j}')/2;
    start = start + length(ind);
end
% LP CONE
for j = 1:1:n_lp
    C{j+n_cones} = -Fbase_x(end,:)';   
end
% FREE CONE
for j = 1:1:n_free
    C{j+n_cones+n_lp}(1) = -Fbase_t(end,j);   
end

% Create dual
y = sdpvar(length(b),1);
cf = [];
Af = [];
Fdual = set([]);
for j = 1:n_cones    
    Ay = reshape(vecA{j}*y,ns(j),ns(j));
    S = C{j}-Ay;
    Fdual = Fdual + lmi(S);    
end
if n_lp > 0
    z = C{n_cones+1}-vecA{n_cones+1}*y ;  
    Fdual = Fdual + set(z);    
    X{end+1} = x;    
end
if n_free > 0
    for j = 1:n_free
        cf = [cf;C{n_cones+n_lp+j}];
        Af = [Af;vecA{n_cones+n_lp+j}];
    end
    Fdual = Fdual + set(0 == cf-Af*y);
end

objdual = b'*y;


