function varargout = geomean2(varargin)
%GEOMEAN2 Nonlinear operator in YALMIP
%
% t = geomean2(X)
%
% For symmatric matrix X, returns det(X)^(1/(2^ceil(log2(length(X)))))
%
% For vector X, returns prod(X)^(1/(2^ceil(log2(length(X)))))
%
% This concave function is monotonically growing in det(P)
%for P>0, so  it can be used for maximizing det(P), 
% or to add lower bound constraints on the determinant.
%
% Note also that the function is the geometric mean of
% the elements (or eigenvalues) if the dimension of 
% X is a power of 2, hence the name GEOMEAN2
%
% See also SDPVAR, SUMK, SUMABSK

% Author Johan Lfberg
% $Id: geomean2.m,v 1.2 2004/12/21 13:22:29 johanl Exp $

% ***************************************************
% This file defines a nonlinear operator for YALMIP
%
% It can take three different inputs
% For double inputs, it returns standard double values
% For sdpvar inputs, it genreates a an internal variable
% When first input is 'model' it generates the epigraph
%
% ***************************************************
switch class(varargin{1})
    
    case 'double' % What is the numerical value of this argument (needed for displays etc)        
        X = varargin{1};
        [n,m] = size(X);
        if min(n,m)==1
            varargout{1} = prod(X)^(1/(2^ceil(log2(length(X)))));
        else
            if issymmetric(X)
                varargout{1} = det(X)^(1/(2^ceil(log2(length(X)))));
            else
                error('GEOMEAN2 only defined for vectors and symmetric matrices');
            end
        end

    case 'sdpvar' % Overloaded operator for SDPVAR objects. Pass on args and save them.
        if is(varargin{1},'symmetric')
            varargout{1} = yalmip('addextendedvariable',mfilename,varargin{:});    
        else
            error('detinv can only be applied to symmetric variables');
        end
        
    case 'char' % YALMIP send 'model' when it wants the epigraph or hypograph
        if isequal(varargin{1},'graph')
            t = varargin{2}; % Second arg is the extended operator variable
            X = varargin{3}; % Third arg and above are the args user used when defining t.           
            varargout{1} = detset(t,X);
            varargout{2} = -1; % Concave operator            
        else           
        end    
    otherwise
end
