function [sol,m,B] = solvesos(F,obj,options,params)
%SOLVESOS Sum of squares decomposition
%
%   [sol,m,B] = solvesos(F,h,options,params) is used for finding
%   SOS-decompositions of polynomials.
%
%    The coefficients of the polynomials are linear w.r.t a set of decision
%    variables 'params' and polynomial with respect to a variable x.
%
%    An extension with a nonlinear parameterization in params
%    is possible. Note though that this gives BMIs or PMIs, solvable (locally)
%    only if PENBMI is installed, or by specifying 'moment' as solver
%    to try to solve the nonconvex semidefinite programming problem using
%    a semidefinite relaxation based on moments.
%
%    The SOS problem can be formulated as
%
%              min h(params)
%
%       subject to  F(i) >(=) 0 or F(i) is SOS w.r.t x
%
%   INPUT
%    F         : SET object with SOS-constrained polynomials and constraints on variables params
%    h         : scalar SDPVAR object (can be [])
%    options   : options structure obtained from SDPSETTINGS (can be [])
%    params    : SDPVAR object defining parametric variables (can be [])
%
%   OUTPUT
%    sol       : Solution diagnostic from SDP problem
%    m         : monomials used in decomposition
%    B         : Gram matrix, p = m'*B*m
%
%   EXAMPLE
%    x = sdpvar(1);solvesos(set(sos(x^4+x^3+1)));                    % Simple decompositions
%    x = sdpvar(1);t = sdpvar(1);solvesos(set(sos(x^4+x^3+1-t)),-t); % Lower bound by maximizing t
%
%   NOTES
%
%    Variables not part of params, but part of non-SOS constraints in F
%    or objective h will automatically be appended to the params list.
%
%    To extract SOS decomposition, use command SOSD
%
%    The field options.sos can be used to tune the SOS-calculations. See HTML help for details
%
%     sos.model          - Kernel or image representation of SOS problem [0|1|2 (0)]
%     sos.newton         - Use Newton polytope to reduce size [0|1 (1)]
%     sos.congruence     - Block-diagonalize using congruence classes [0|1|2 (2)]
%     sos.inconsistent   - Remove diagonal-inconsistent monomials [0|1|2 (0)]
%     sos.clean          - Remove monomials with coefficients < clean [real > 0 (1e-4)]
%     sos.traceobj       - Minimize trace of Gram matrix in problems without objective function [0|1 (0)]
%
% See also SOSD, SDPSETTINGS, SOLVEMOMENT, SDPVAR, SDISPLAY

% Time YALMIP
yalmip_time = clock;

% Check #inputs
if nargin<4
    params = [];
    if nargin<3
        options = sdpsettings;
        if nargin<2
            obj = [];
            if nargin<1
                error('At-least one argument needed')
            end
        end
    end
end

% Lazy syntax (not official)
if nargin==1 & isa(F,'sdpvar')
    F = set(sos(F));
end

if ~any(is(F,'sos'))
    error('At-least one constraint should be an SOS constraints!');
end

% Extract all SOS constraints and place them in a cell p
p = [];
for i = 1:length(F)
    if is(F(i),'sos')
        pi = sdpvar(F(i));
        p{end+1} = pi;
    end
end

% ************************************************
% Get the parametric constraints
% ************************************************
F_original = F;
F_parametric = F(find(~is(F,'sos')));
if isempty(F_parametric)
    F_parametric = set([]);
end

if isempty(options)
    options = sdpsettings;
end

if ~isempty(params)
    if ~isa(params,'sdpvar')
        error('Fourth argment should be a SDPVAR variable or empty')
    end
end

if options.verbose>0;
    disp('-------------------------------------------------------------------------');
    disp('YALMIP SOS module started...');
    disp('-------------------------------------------------------------------------');
end

% ************************************************
% INITIALIZE SOS-DECOMPOSITIONS SDP CONSTRAINTS
% ************************************************
F_sos = set([]);

% ************************************************
% FIGURE OUT ALL PARAMETRIC VARIABLES
% ************************************************
AllVariables =  uniquestripped([depends(obj) depends(F_original)]);
ParametricVariables = uniquestripped([depends(obj) depends(F_parametric) depends(params)]);
ParametricVariables = intersect(ParametricVariables,AllVariables);
MonomVariables = setdiff(AllVariables,ParametricVariables);
params = recover(ParametricVariables);

if isempty(MonomVariables)
    error('No independent variables? Perhaps you added a constraint set(p(x)) when you meant set(sos(p(x)))');
end

if options.verbose>0;disp(['Detected ' num2str(length(ParametricVariables)) ' parametric variables and ' num2str(length(MonomVariables)) ' independent variables.']);end

% ************************************************
% ANY BMI STUFF
% ************************************************
NonLinearParameterization = 0;
if ~isempty(ParametricVariables)
    monomtable = yalmip('monomtable');
    ParametricMonomials = monomtable(uniquestripped([getvariables(obj) getvariables(F_original)]),ParametricVariables);
    if any(sum(abs(ParametricMonomials),2)>1)
        NonLinearParameterization = 1;
    end
end

% ************************************************
% DISPLAY WHAT WE FOUND
% ************************************************
if options.verbose>0 & ~isempty(F_parametric)
    nLP = 0;
    nEQ = 0;
    nLMI = sum(full(is(F_parametric,'lmi')) &  full(~is(F_parametric,'element-wise'))); %FULL due to bug in ML 7.0.1
    for i = 1:length(F_parametric)
        if is(F_parametric,'element-wise')
            nLP = nLP + numel(F_parametric(i));
        end
        if is(F_parametric,'equality')
            nEQ = nEQ + numel(F_parametric(i));
        end
    end
    disp(['Detected ' num2str(full(nLP)) ' linear inequalities, ' num2str(full(nEQ)) ' equality constraints and ' num2str(full(nLMI)) ' LMIs.']);
end


% ************************************************
% IMAGE OR KERNEL REPRESENTATION?
% ************************************************
switch options.sos.model
    case 0
        constraint_classes = constraintclass(F);
        noCOMPLICATING = ~any(ismember([4 7 8 9 10 12 13 14],constraint_classes));% ~any(is(F,'lmi')) & ~any(is(F,'element-wise')) & ~any(is(F,'socc'));
        if noCOMPLICATING & ~NonLinearParameterization
            options.sos.model = 1;
            if options.verbose>0;disp('Using kernel representation (model 1).');end
        else
            options.sos.model = 2;
            if options.verbose>0;disp('Using image representation (model 2) since SOCPs or integer data was found.');end
        end
    case 1
        if any(is(F,'socc'))
            if options.verbose>0;disp('Switching to image model due to SOCP constraints (not supported in kernel model).');end
            options.sos.model = 2;
        end
        if NonLinearParameterization
            if options.verbose>0;disp('Switching to image model due to nonlinear parameterization (not supported in kernel model).');end
            options.sos.model = 2;
        end
    case 3

    otherwise
end

% ************************************************
% DIAGONAL INCONSITENCY HARD FOR PARAMETRIC MODEL
% ************************************************
if ~isempty(params) & options.sos.inconsistent
    if options.verbose>0;disp('Turning off inconsistency based reduction (not supported in parametric models).');end
    options.sos.inconsistent = 0;
end

if ~isempty(obj)
    options.sos.traceobj = 0;
end

parobj = obj;
obj    = [];

for constraint = 1:length(p)
    % *********************************************
    % FIND THE VARIABLES IN p, SORT, GET UNIQUE ETC
    % *********************************************
    if options.verbose>1;disp(['Creating SOS-description ' num2str(constraint) '/' num2str(length(p)) ]);end

    pVariables = depends(p{constraint});
    AllVariables = uniquestripped([pVariables ParametricVariables]);
    MonomVariables = setdiff1D(pVariables,ParametricVariables);
    x = recover(MonomVariables);
    z = recover(AllVariables);
    MonomIndicies = find(ismember(AllVariables,MonomVariables));
    ParametricIndicies = find(ismember(AllVariables,ParametricVariables));

    % *********************************************
    % MONOMIAL GENERATION AND REDUCTION
    % *********************************************
    [exponent_p,p_base] = getexponentbase(p{constraint},z);

    %...we have left the pure YALMIP specific code now

    % *********************************************
    % STUPID PROBLEM WITH ODD HIGHEST POWER?...
    % *********************************************
    if isempty(ParametricIndicies)
        max_degrees = max(exponent_p(:,MonomIndicies),[],1);
        bad_max = any(max_degrees-fix((max_degrees/2))*2);
        if bad_max
            for i = 1:length(p)
                B{i}=[];
            end
            sol.yalmiptime = etime(clock,yalmip_time);
            sol.solvertime = 0;
            sol.info = yalmiperror(1,'YALMIP');
            sol.problem = 2;
            return
        end
    end

    % *********************************************
    % Can we make a variable smart change (no smart strategy currently)
    % *********************************************
    %[exponent_p_monoms,varchange] =
    %variablechange(exponent_p(:,MonomIndicies));
    exponent_p_monoms = exponent_p(:,MonomIndicies);
    varchange = ones(1,size(MonomIndicies,2));

    % *********************************************
    % Unique monoms (copies due to parametric terms)
    % *********************************************
    exponent_p_monoms = unique(exponent_p_monoms,'rows');

    if options.sos.reuse & constraint > 1 & isequal(previous_exponent_p_monoms,exponent_p_monoms)
        % We don't have to do anything, exponent
        if options.verbose>1;disp(['Re-using all candidate monomials (same problem structure)']);end
    else

        % *********************************************
        % CORRELATIVE SPARSITY PATTERN
        % *********************************************
        [C,csclasses] = corrsparsity(exponent_p_monoms,options);

        % *********************************************
        % GENERATE MONOMIALS
        % *********************************************
        exponent_m = monomialgeneration(exponent_p_monoms,csclasses);

        % *********************************************
        % REDUCE #of MONOMIALS
        % *********************************************
        exponent_m = monomialreduction(exponent_m,exponent_p_monoms,options,csclasses);

        % *********************************************
        % BLOCK PARTITION THE MONOMIALS BY CONGRUENCE
        % *********************************************
        N = congruenceblocks(exponent_m,exponent_p_monoms,options,csclasses);

        % *********************************************
        % REDUCE FURTHER BY EXPLOITING BLOCK-STRUCTURE
        % *********************************************
        N = blockmonomialreduction(exponent_p_monoms,N,options);

        % *********************************************
        % PREPARE FOR SDP FORMULATION BY CALCULATING ALL
        % POSSIBLE MONOMIAL PRODUCS IN EACH BLOCK
        % *********************************************
        [exponent_m2,N_unique] = monomialproducts(N);

        % *********************************************
        % CHECK FOR BUG/IDIOT PROBLEMS IN FIXED PROBLEM
        % *********************************************
        if isempty(ParametricIndicies)
            if ~isempty(setdiff(exponent_p_monoms,N_unique(:,3:end),'rows'))
                for i = 1:length(p)
                    B{i}=[];
                end
                sol.problem = 2;
                sol.info = yalmiperror(1,'YALMIP');
                warning('Problem is trivially infeasible (odd highest power?)');
                return
            end
        end
    end

    previous_exponent_p_monoms = exponent_p_monoms;

    % *********************************************
    % GENERATE DATA FOR SDP FORMULATIONS
    % *********************************************
    p_base_parametric = parameterizedbase(p{constraint},z,params,ParametricIndicies,exponent_p,p_base);
    [BlockedA{constraint},Blockedb{constraint}] = generate_kernel_representation_data(N,N_unique,exponent_m2,exponent_p,p{constraint},options,p_base_parametric,ParametricIndicies,MonomIndicies,constraint==1);

    % SAVE FOR LATER
    BlockedN{constraint} = N;
    Blockedx{constraint} = x;
    Blockedvarchange{constraint}=varchange;
end

% *********************************************
% And now get the SDP formulations
%
% The code above has generated matrices A and b
% in AQ == b(parametric)
%
% We use these to generate kernel or image models
% *********************************************
switch options.sos.model

    case 1
        % To get the primal kernel representation, we simply use
        % the built-in dualization module!
        % First, write the problem in primal kernel format
        traceobj = 0;
        F = F_parametric;
        for i = 1:length(Blockedb)
            res = 0;
            for j = 1:length(BlockedA{i})
                n = sqrt(size(BlockedA{i}{j},2));
                BlockedQ{i}{j} = sdpvar(n,n);
                F = F + set(BlockedQ{i}{j} > 0);
                res = res + BlockedA{i}{j}*BlockedQ{i}{j}(:);
                traceobj = traceobj + trace(BlockedQ{i}{j});
            end
            F = F + set(res == Blockedb{i});
        end
        % And get the primal model of this
        if isempty(parobj)
            if options.sos.traceobj
                [F,obj,Primal_matrices,Free_variables] = dualize(F,traceobj);
            else
                [F,obj,Primal_matrices,Free_variables] = dualize(F,[]);
            end
        else
            [F,obj,Primal_matrices,Free_variables] = dualize(F,parobj);
        end
        % In dual mode, we maximize
        obj = -obj;

    case 2
        % *********************************
        % IMAGE REPRESENTATION
        % Needed for nonlinearly parameterized problems
        % More efficient in some cases
        % *********************************
        g = [];
        vecQ = [];
        for i = 1:length(BlockedA)
            q = [];
            A = [];
            for j = 1:length(BlockedA{i})
                n = sqrt(size(BlockedA{i}{j},2));
                BlockedQ{i}{j} = sdpvar(n,n);
                q = [q;reshape(BlockedQ{i}{j},n^2,1)];
                A = [A BlockedA{i}{j}];
            end
            vecQ = [vecQ;q];
            g = [g;Blockedb{i}-A*q];
        end

        g_vars = getvariables(g);
        q_vars = getvariables(vecQ);
        x_vars = setdiff(g_vars,q_vars);

        base = getbase(g);       
        if isempty(x_vars)
            A = base(:,1);base = base(:,2:end);
            B = (base(:,ismember(g_vars,q_vars)));
            Bnull = sparsenull(B);
            t = sdpvar(size(Bnull,2),1);
            imQ = -B\A+Bnull*t;
        else
            A = base(:,1);base = base(:,2:end);
            C = base(:,ismember(g_vars,x_vars));
            B = (base(:,ismember(g_vars,q_vars)));
            [Bnull,Q1,R1] = sparsenull(B);Bnull(abs(Bnull) < 1e-12) = 0;
            t = sdpvar(size(Bnull,2),1);                        
            imQ = -Q1*(R1'\(A+C*recover(x_vars)))+Bnull*t;           
        end
        notUsed = find(sum(abs(B),2)==0);
        if ~isempty(notUsed)
            F_parametric = F_parametric + set(g(notUsed)==0);
        end
        F_sos = set([]);
        obj = 0;
        for i = 1:length(BlockedQ)
            for j = 1:size(BlockedQ{i},2)
                Q_old = BlockedQ{i}{j};
                Q_old_vars = getvariables(Q_old);
                Q_old_base = getbase(Q_old);
                in_this = [];
                for k = 1:length(Q_old_vars)
                    in_this = [in_this;find(Q_old_vars(k)==q_vars)];
                end
                Q_new = Q_old_base(:,1) + Q_old_base(:,2:end)*imQ(in_this);
                Q_new = reshape(Q_new,length(BlockedQ{i}{j}),length(BlockedQ{i}{j}));
                obj = obj+trace(Q_new);
                if ~isa(Q_new,'double')
                    F_sos = F_sos + set(Q_new > 0);
                elseif min(eig(Q_new))<-1e-8
                    sol.yalmiptime = etime(clock,yalmip_time);
                    sol.solvertime = 0;
                    sol.problem = 2;
                    sol.info = yalmiperror(1,'YALMIP');
                    %warning('Problem is trivially infeasible (negative constant?)');
                    return
                end
                BlockedQ{i}{j} = Q_new;
            end
        end

        F = F_parametric + F_sos;

        if isa(obj,'double') | (options.sos.traceobj == 0)
            obj = [];
        end

        if ~isempty(parobj)
            obj = parobj;
        end

    case 3

        % Some special code for ther low-rank model in SDPLR
        % Experimental code, not official yet
        allb = [];
        allA = [];
        K.s = [];
        for i = 1:length(Blockedb)
            allb = [allb;Blockedb{i}];
            Ai = [];
            for j = 1:size(BlockedA{i},2)
                Ai = [Ai BlockedA{i}{j}];
                K.s = [K.s sqrt(size(BlockedA{i}{j},2))];
            end
            %blkdiag bug in 7.0...
            [n1,m1] = size(allA);
            [n2,m2] = size(Ai);
            allA = [allA spalloc(n1,m2,0);spalloc(n2,m1,0) Ai];
        end
        options.solver = 'sdplr';
        z = recover(ParametricVariables)
        start = size(BlockedA,2)+1;
        Mi = [];
        for i = 1:length(allb)
            if isa(allb(i),'sdpvar')
                [Qi,ci,fi,xi,infoi] = quaddecomp(allb(i),z);
            else
                Qi = zeros(length(z));
                ci = zeros(length(z),1);
                fi = allb(i);
            end
            Z = -[fi ci'/2;ci/2 Qi];
            Mi = [Mi;Z(:)'];
        end
        K.s = [K.s length(z)+1];
        zeroRow = zeros(1,size(allA,2));
        allA = [allA Mi;zeroRow 1 zeros(1,K.s(end)^2-1)];
        b = zeros(size(allA,1),1);b(end) = 1;
        y = sdpvar(length(b),1);
        CminusAy = -allA'*y;
        start = 1;

        % Get the cost, expressed in Z
        [Qi,ci,fi,xi,infoi] = quaddecomp(parobj,z);
        C = [fi ci'/2;ci/2 Qi];
        F = set([]);
        for i = 1:length(K.s)
            if i<length(K.s)
                F = F + set(reshape(CminusAy(start:start+K.s(i)^2-1),K.s(i),K.s(i)));
            else
                F = F + set(reshape(C(:) + CminusAy(start:start+K.s(i)^2-1),K.s(i),K.s(i)));
            end
            start = start + K.s(i)^2;
        end
        obj = -b'*y;

        options.sdplr.forcerank = ceil(K.s/2);
        options.sdplr.forcerank(end) = 1;
        options.sdplr.feastol = 1e-7;

    otherwise
end

% **********************************************
% SOLVE SDP
% **********************************************
sol =  solvesdp(F,obj,options);

switch sol.problem
    case {0,1,2,3,4,5} % Well, it didn't f**k up completely at-least
        % *********************************************
        % RECOVER PRIMAL AND DUAL VARIABLES
        % *********************************************
        switch options.sos.model

            case 1
                % Recover all parametric variables
                if length(Free_variables)>0
                    setsdpvar(Free_variables,dual(F(end)));
                end
                for i = 1:length(Primal_matrices)
                    setsdpvar(Primal_matrices{i},dual(F(i)));
                end
                % Recover the Gramians
                for constraint = 1:length(p)
                    Qtemp = [];
                    for i = 1:length(BlockedQ{constraint})
                        Qtemp = blkdiag(Qtemp,double(BlockedQ{constraint}{i}));
                    end
                    Q{constraint} = Qtemp;
                end

            case 2

                for constraint = 1:length(p)
                    Qtemp = [];
                    for i = 1:length(BlockedQ{constraint}) % Loop over congruence partition
                        Qtemp = blkdiag(Qtemp,double(BlockedQ{constraint}{i}));
                    end
                    Q{constraint} = full(Qtemp);
                end

            case 3
                % If SDPLR was sucessful, the last dual should have rank 1
                Z = dual(F(end));
                i = 1e-16;
                [R,fail] = chol(Z);
                while fail & i<10
                    i = i*100;
                    [R,fail] = chol(Z+i*eye(length(Z)));
                end
                Z = R(1,2:end)';
                setsdpvar(z,Z);
                start = 1;
                for constraint = 1:length(p)
                    Qi = [];
                    for j = 1:length(BlockedA{constraint})
                        Qi = blkdiag(Qi,dual(F(start)));
                        start = start + 1;
                    end
                    Q{constraint} = Qi;
                end

            otherwise
        end

        % *********************************************
        % GENERATE MONOMIALS IN SOS-DECOMPOSITION
        % *********************************************
        B = Q;
        for constraint = 1:length(p)

            if constraint > 1 & isequal(BlockedN{constraint},BlockedN{constraint-1}) & isequal(Blockedx{constraint},Blockedx{constraint-1}) & isequal(Blockedvarchange{constraint},Blockedvarchange{constraint-1})
                monoms{constraint} = monoms{constraint-1};
            else
                monoms{constraint} = [];
                totalN{constraint} = [];
                N = BlockedN{constraint};
                x = Blockedx{constraint};
                for i = 1:length(N)
                    % Original variable
                    for j = 1:size(N{i},1)
                        N{i}(j,:)=N{i}(j,:).*Blockedvarchange{constraint};
                    end
                    monoms{constraint} = [monoms{constraint};recovermonoms(N{i},x)];
                end
            end


            % For small negative eigenvalues
            % this is a good quick'n'dirty approximation
            % Improve...use shifted eigenvalues and chol or what ever...
            if ~any(any(isnan(B{constraint})))
                [U,S,V]=svd(B{constraint});
                R = sqrt(S)*V';
                h0 = R*monoms{constraint};
            else
                if options.verbose>0;disp('FAILURE : SOS decomposition not available.');end
                if options.verbose>0;disp('The reason is probably that you are using a solver that does not deliver a dual (LMILAB)');end;
                if options.verbose>0;disp('Use sdsettings(''sos.model'',2) to solve this.');end;
                if options.verbose>0;disp('or use another solver (SDPT3, SEDUMI,...)');end;


                h0 = repmat(nan,size(monoms{constraint},1),1);
            end

            if isa(h0,'sdpvar')
                h{constraint} = clean(R*monoms{constraint},options.sos.clean);
                h{constraint} = h{constraint}(find(h{constraint}));
            else
                h{constraint} = h0;
            end
        end

        % Argh, changed syntax. lazy hack...
        if length(p)==1
            setsos(p{1},h{1},ParametricVariables);
            B = B{1};
            m = monoms{1};
        else
            F_sos = F_original(find(is(F_original,'sos')));
            for i = 1:length(F_sos)
                setsos(sdpvar(F_sos(i)),h{i},ParametricVariables);
            end
            m = monoms;
        end

    otherwise
        B = [];
        m = [];
end

% Don't need these outside
yalmip('cleardual')

% Done with YALMIP, this is the time it took, minus solver
if ~isfield(sol,'solvertime')
    sol.solvertime = 0;
end
sol.yalmiptime = etime(clock,yalmip_time)-sol.solvertime;


function p_base_parametric = parameterizedbase(p,z, params,ParametricIndicies,exponent_p,p_base)

% Check for linear parameterization
parametric_basis = exponent_p(:,ParametricIndicies);
if all(sum(parametric_basis,2)==0)
    p_base_parametric = full(p_base(:));
    return
end
if all(sum(parametric_basis,2)<=1)
    p_base_parametric = full(p_base(:));
    n = length(p_base_parametric);
    ii = [];
    vars = [];
    js = sum(parametric_basis,1);
    for i = 1:size(parametric_basis,2)
        if js(i)
            %    j = js(i);
            j = find(parametric_basis(:,i));
            %if js(i)%~isempty(j)
            ii = [ii j(:)'];
            %vars = [vars repmat(i,1,length(j))];
            vars = [vars repmat(i,1,js(i))];
        end
    end
    k = setdiff1D(1:n,ii);
    if isempty(k)
        p_base_parametric = p_base_parametric.*sparse(ii,repmat(1,1,n),params(vars));
    else
        pp = params(vars); % Must do this, bug in ML 6.1 (x=sparse(1);x([1 1]) gives different result in 6.1 and 7.0!)
        p_base_parametric = p_base_parametric.*sparse([ii k(:)'],repmat(1,1,n),[pp(:)' ones(1,1,length(k))]);
    end
else
    % Bummer, nonlinear parameterization sucks...
    for i = 1:length(p_base)
        j = find(exponent_p(i,ParametricIndicies));
        if ~isempty(j)
            temp = p_base(i);

            for k = 1:length(j)
                if exponent_p(i,ParametricIndicies(j(k)))==1
                    temp = temp*params(j(k));
                else
                    temp = temp*params(j(k))^exponent_p(i,ParametricIndicies(j(k)));
                end
            end
            xx{i} = temp;
        else
            xx{i} = p_base(i);
        end
    end
    p_base_parametric = stackcell(sdpvar(1,1),xx)';
end









function [A,b] = generate_kernel_representation_data(N,N_unique,exponent_m2,exponent_p,p,options,p_base_parametric,ParametricIndicies,MonomIndicies,FirstRun)

persistent saveData

exponent_p_parametric = exponent_p(:,ParametricIndicies);
exponent_p_monoms = exponent_p(:,MonomIndicies);
pcoeffs = getbase(p);
if any(exponent_p_monoms(1,:))
    pcoeffs=pcoeffs(2:end); % No constant term in p
end
b = [];

parametric = full((~isempty(ParametricIndicies) & any(any(exponent_p_parametric))));

% For problems with a lot of similar cones, this saves some time
reuse = 0;
if ~isempty(saveData) & isequal(saveData.N,N) & ~FirstRun
    n = saveData.n;
    ind = saveData.ind;
    if  isequal(saveData.N_unique,N_unique) & isequal(saveData.exponent_m2,exponent_m2)% & isequal(saveData.epm,exponent_p_monoms)
        reuse = 1;
    end
else
    % Congruence partition sizes
    for k = 1:size(N,1)
        n(k) = size(N{k},1);
    end
    % Save old SOS definition
    saveData.N = N;    
    saveData.n = n;    
    saveData.N_unique = N_unique;
    saveData.exponent_m2 = exponent_m2;
    saveData.N_unique = N_unique;
end

if reuse & options.sos.reuse
    % Get old stuff
    allj = [];
    used_in_p = zeros(size(exponent_p_monoms,1),1);
    for i = 1:size(N_unique,1)
        monom = N_unique(i,3:end);
        j = findrows(exponent_p_monoms,monom);
        if isempty(j)
            b = [b 0];
            allj(end+1,1) = 0;
        else
            used_in_p(j) = 1;
            allj(end+1,1:length(j)) = j(:)';
        end
    end
    ind = saveData.ind;
else
    allj = [];
    used_in_p = zeros(size(exponent_p_monoms,1),1);
    if size(exponent_m2{1},2)==2 % Stupid set(sos(parametric)) case      
        ind = spalloc(size(vecBdual,1),1,0);
        ind(1)=1;
        allj = 1:size(exponent_p_monoms,1);
        used_in_p = ones(size(exponent_p_monoms,1),1);
    else
        % To speed up some searching, we random-hash data
        hash = randn(size(exponent_p_monoms,2),1);
        for k = 1:length(exponent_m2)
            exp_hash{k} = sparse((exponent_m2{k}(:,3:end)))*hash; % SPARSE NEEDED DUE TO STRANGE NUMERICS IN MATLAB ON 0s (the stuff will differ on last bit in hex format)
        end

        p_hash = exponent_p_monoms*hash;
                        
        ind = spalloc(size(N_unique,1),sum(n.^2),0);
        for i = 1:size(N_unique,1)
            monom = N_unique(i,3:end);
            monom_hash = sparse(monom)*hash;
            LHS = 0;
            start = 0;
            for k = 1:size(N,1)
                j = find(exp_hash{k} == monom_hash);             
                if ~isempty(j)
                    pos=exponent_m2{k}(j,1:2);
                    ns = pos(:,1);
                    ms = pos(:,2);                   
                    indicies = ns+(ms-1)*n(k);                
                    ind(i,indicies+start) = ind(i,indicies+start) + 1;
                end              
                start = start + n(k)^2;
            end         

            j = find(p_hash == monom_hash);
          
            if isempty(j)
                allj(end+1,1) = 0;
            else
                used_in_p(j) = 1;
                allj(end+1,1:length(j)) = j(:)';
            end
        end
    end
end
saveData.ind = ind;

% Some parametric terms in p(x,t) do not appear in v'Qv
% So these have to be added 0*Q = b
not_dealt_with  = find(used_in_p==0);
while ~isempty(not_dealt_with)
    j = findrows(exponent_p_monoms,exponent_p_monoms(not_dealt_with(1),:));
    allj(end+1,1:length(j)) = j(:)';
    used_in_p(j) = 1;
    not_dealt_with  = find(used_in_p==0);   
    ind(end+1,1)=0;
end

if parametric
    % Inconsistent behaviour in MATLAB
    if size(allj,1)==1
        uu = [0;p_base_parametric];
        b = sum(uu(allj+1))';
    else
        uu = [0;p_base_parametric];
        b = sum(uu(allj+1),2)';
    end
else
    uu = [0;pcoeffs(:)];
    b = sum(uu(allj+1),2)';
end

b = b';
dualbase = ind;
j = 1;
A = cell(size(N,1),1);
for k = 1:size(N,1)
    A{k} = dualbase(:,j:j+n(k)^2-1);
    j = j + n(k)^2;
end


function [Z,Q1,R] = sparsenull(A)

[Q,R] = qr(A');
n = max(find(sum(abs(R),2)));
Q1 = Q(:,1:n);
R = R(1:n,:);
Z = Q(:,n+1:end); % New basis